/* ArduinosInSpaceDemo
   This sketch demonstrates some of the input and output capabilities
   of ArduinosInSpace. It controls some simple aspects of flight,
   and uses some LEDs to display output from boolean data channels.

   Hardware:
   - Two toggle switches, connected to pins 0 and 2. These will switch
     IFF and EMCON
     mode on and off, respectively.
   - A push button switch attached to pin 1. Pushing this will burn the
     main engine.
   - LEDs connected to pins 8, 9, and 10. These will indicate status of
     EMCON, main engine and Autopilot respectively.
   Software:
   - The Bounce2 library is used to simplify button debouncing. Install
     it using the Arduino Library Manager, or from 
     https://github.com/thomasfredericks/Bounce2
*/
#include <Bounce2.h>
#include <ArduinosInSpace.h>

const int iffTogglePin = 0;
const int engineButtonPin = 1;
const int emconTogglePin = 2;
Bounce iffToggle = Bounce();
Bounce engineButton = Bounce();
Bounce emconToggle = Bounce();

const int emconLEDPin = 8;
const int engineLEDPin = 9;
const int autopilotLEDPin = 10;

// Create ObjectsInSpace object, passing it our
// serial device and total number of *request* channels
ObjectsInSpace OIS(Serial, 3);

void setup() {
  pinConfiguration();

  // Open serial connection and perform handshaking
  Serial.begin(9600);
  OIS.begin();

  // Perform synchronisation with the game
  channelSynchronisation();

  // Activate the game interface
  OIS.activate();
}

void loop() {
  // These functions check the state of our
  // buttons and send commands if necessary.
  iffCheck();
  engineCheck();
  emconCheck();

  // The update loop checks for and processes serial data
  OIS.update();
}

void pinConfiguration() {
  pinMode(iffTogglePin, INPUT_PULLUP);
  pinMode(engineButtonPin, INPUT_PULLUP);
  pinMode(emconTogglePin, INPUT_PULLUP);

  pinMode(emconLEDPin, OUTPUT);
  pinMode(engineLEDPin, OUTPUT);
  pinMode(autopilotLEDPin, OUTPUT);

  // The Bounce2 library takes care of input debouncing for us.
  iffToggle.attach(iffTogglePin);
  iffToggle.interval(5);
  engineButton.attach(engineButtonPin);
  engineButton.interval(5);
  emconToggle.attach(emconTogglePin);
  emconToggle.interval(5);
}

void channelSynchronisation() {
  // As part of the synchronisation process, the game requires
  // us to register the commands this device will be sending.
  // The registerCommand() function can does this.
  OIS.registerCommand(BURN_MAIN_ENGINE);
  OIS.registerCommand(STOP_MAIN_ENGINE);
  OIS.registerCommand(EMCON_ON);
  OIS.registerCommand(EMCON_OFF);
  OIS.registerCommand(TURN_ON_IFF);
  OIS.registerCommand(TURN_OFF_IFF);
  // The registerBool command is used to request a data channel
  // back from the game. This channel will send boolean data
  // whenever the state changes, as well as periodically at
  // other times.
  OIS.registerBool(EMCON_MODE, emconCallback);
  OIS.registerBool(MAIN_ENGINE_BURNING, engineCallback);
  OIS.registerBool(AUTO_PILOT_ENGAGED, autopilotCallback);

}

void iffCheck() {
  iffToggle.update();
  if (iffToggle.fell()) {
    OIS.sendCommand(TURN_ON_IFF);
  } else if (iffToggle.rose()) {
    OIS.sendCommand(TURN_OFF_IFF);
  }
}

void engineCheck() {
  engineButton.update();
  if (engineButton.fell()) {
    OIS.sendCommand(BURN_MAIN_ENGINE);
  } else if (engineButton.rose()) {
    OIS.sendCommand(STOP_MAIN_ENGINE);
  }
}

void emconCheck() {
  emconToggle.update();
  if (emconToggle.fell()) {
    OIS.sendCommand(EMCON_ON);
  } else if (emconToggle.rose()) {
    OIS.sendCommand(EMCON_OFF);
  }
}

// ArduinosInspace will call this function whenever EMCON status data is
// received from the game. We light the LED if the state is true, to
// show EMCOn is active.
void emconCallback(int channel, bool data) {
  if (data == 0) {
    digitalWrite(emconLEDPin, LOW);
  } else {
    digitalWrite(emconLEDPin, HIGH);
  }
}

// This function will be called whenever the game sends the status of the
// main engine. We light the LED if the status is true (engine is burning).
void engineCallback(int channel, bool data) {
  if (data == 0) {
    digitalWrite(engineLEDPin, LOW);
  } else {
    digitalWrite(engineLEDPin, HIGH);
  }
}

// ArduinosInSpace calls this function whenever we receive data about
// the Autopilot status. If the Autopilot is active, we light the LED.
void autopilotCallback(int channel, bool data) {
  if (data == 0) {
    digitalWrite(autopilotLEDPin, LOW);
  } else {
    digitalWrite(autopilotLEDPin, HIGH);
  }
}
