#include "ArduinosInSpace.h"
#include "AISRequest.h"

ObjectsInSpace::ObjectsInSpace(Stream &serial, int numRequests)
{
  _serial = &serial;
  _numRequests = numRequests;
  _commandCallbacks = new AISRequest*[numRequests];
  _callbackidx = 0;
  _streamState = handshaking;
}

bool ObjectsInSpace::begin()
{
  // TODO: Sending OIS SYN packets / waiting for ACK packets here.
  // return false if we got a NAK.
  while (_streamState == handshaking) {
    // Beta doesn't follow the docs. Hax.
    // _serial->print("SYN=1\n");
    _serial->print("451\n");
    delay(1000);
    update();
  }
}

// Channel names are long, and speed is not important for register
// commands. So we use multiple prints here to keep the required
// send buffer size small.
int ObjectsInSpace::registerCommand(OISCommand command)
{
  if (command.channel < 0 || command.channel > maxCommand) {
    return -1;
  } else {
    _serial->print("CMD=");
    _serial->print(command.name);
    _serial->print(",");
    _serial->print(command.channel);
    _serial->print("\n");
    return 0;
  }
}

int ObjectsInSpace::registerBool(OISCommand command, boolCallback callback)
{
  if (_validateCommand(command)) {
    _sendRequest(boolRequestType, command);
    _commandCallbacks[_callbackidx] = new boolRequest(callback);
    _callbackidx++;
    return (_callbackidx-1);
  } else {
    return -1;
  }
}

int ObjectsInSpace::registerInt(OISCommand command, intCallback callback)
{
  if (_validateCommand(command)) {
    _sendRequest(intRequestType, command);
    _commandCallbacks[_callbackidx] = new intRequest(callback);
    _callbackidx++;
    return (_callbackidx-1);
  } else {
    return -1;
  }
}

int ObjectsInSpace::registerFloat(OISCommand command, floatCallback callback)
{
  if (_validateCommand(command)) {
    _sendRequest(floatRequestType, command);
    _commandCallbacks[_callbackidx] = new floatRequest(callback);
    _callbackidx++;
    return (_callbackidx-1);
  } else {
    return -1;
  }
}

int ObjectsInSpace::registerBoolRaw(OISCommand command, rawCallback callback)
{
  if (_validateCommand(command)) {
    _sendRequest(boolRequestType, command);
    _commandCallbacks[_callbackidx] = new rawRequest(callback);
    _callbackidx++;
    return (_callbackidx-1);
  } else {
    return -1;
  }
}

int ObjectsInSpace::registerIntRaw(OISCommand command, rawCallback callback)
{
  if (_validateCommand(command)) {
    _sendRequest(intRequestType, command);
    _commandCallbacks[_callbackidx] = new rawRequest(callback);
    _callbackidx++;
    return (_callbackidx-1);
  } else {
    return -1;
  }
}

int ObjectsInSpace::registerFloatRaw(OISCommand command, rawCallback callback)
{
  if (_validateCommand(command)) {
    _sendRequest(floatRequestType, command);
    _commandCallbacks[_callbackidx] = new rawRequest(callback);
    _callbackidx++;
    return (_callbackidx-1);
  } else {
    return -1;
  }
}

float ObjectsInSpace::toFloat(int data)
{
  return (data * 0.01);
}

void ObjectsInSpace::activate()
{
  _serial->print("ACT\n");
  _serial->print("DBG=ArduinosInSpace device successfully connected!\n");
  _streamState = active;
}

void ObjectsInSpace::sendCommand(OISCommand command)
{
  snprintf(sendBuffer, bufSize, "EXC=%i\n", command.channel);
  _serial->print(sendBuffer);
}

void ObjectsInSpace::sendDebug(char* message)
{
  // Just send message to the game
  return true;
}

void ObjectsInSpace::update()
{
  _readSerial();
}

void ObjectsInSpace::_readSerial()
{
  #ifdef OIS_DEBUG
  if (_serial->available() >= SERIAL_RX_BUFFER_SIZE
      && _errorCallback != NULL) {
    _errorCallback(OISSerialOverflow);
  }
  #endif
  while (_serial->available()) {
    recvBuffer[recvidx] = _serial->read();
    if (recvBuffer[recvidx] == '\n') {
      recvBuffer[recvidx] = '\0';
      _parseSerial();
      recvidx = 0;
    } else {
      recvidx++;
      if (recvidx == bufSize) {
	// We've overflowed the buffer.
	// TODO: Add a dirty flag or something to invalidate this command.
	recvidx = 0;
      }
    }
  }
}

void ObjectsInSpace::_parseSerial()
{
  // At this point, recvBuffer should contain a null-terminated string.
  // The possible contents depend on what state we're in.
  // handshaking: Only ACK or DEN.
  // syncing: The server doesn't send anything in this state.
  // active: key=value
  // The key will be an int representing the channel. Value
  // will be an int, either 0/1 for boolean, an integer value,
  // or a fixed point representation of a float with two digits
  // of precision.
  switch (_streamState) {
  case active:
    int commandIndex;
    int payload;
    char *token;
    token = strtok(recvBuffer, "=");
    commandIndex = atoi(token);
    token = strtok(NULL, "=");
    payload = atoi(token);

    // Execute the callback indicated by commandIndex.
    // Pass it payload.
    // TODO: Sanity checking here. Bounds checks.
    // Make sure there's actually a function at this index.
    _commandCallbacks[commandIndex]->runCallback(commandIndex, payload);
    break;
  case handshaking:
    // Beta doesn't follow docs, should be ACK
    if (strcmp(recvBuffer, "452\r")) {
      _streamState = syncing;
    } else if (strcmp(recvBuffer, "DEN")) {
      // TODO: error condition?
    }
  }
}

bool ObjectsInSpace::_validateCommand(OISCommand command)
{
  if (command.channel < 0 || command.channel > maxCommand) {
    return false;
  } else if (_callbackidx >= _numRequests) {
    return false;
  } else {
    return true;
  }
}

void ObjectsInSpace::_sendRequest(OISRequest request, OISCommand command)
{
  switch(request) {
  case boolRequestType:
    _serial->print("NIB=");
    break;
  case intRequestType:
    _serial->print("NIN=");
    break;
  case floatRequestType:
    _serial->print("NIF=");
    break;
  default:
    return;
  }
  _serial->print(command.name);
  _serial->print(",");
  _serial->print(_callbackidx);
  _serial->print("\n");
}

#ifdef OIS_DEBUG
void errorHandler(OISErrorCallback callback)
{
  _errorCallback = callback;
}
#endif
