#ifndef ArduinosInSpace_h
#define ArduinosInSpace_h

#include <Arduino.h>

#include "AISRequest.h"
#include "AISUtil.h"
#include "ArduinosInSpaceCommands.h"
#include "ArduinosInSpaceRequests.h"

// Only used for sending EXC commands.
const byte bufSize = 10;


typedef void (*commandCallback)(int command, int payload);

// TODO: Fix this.
const int maxCommand = 300;

class ObjectsInSpace
{
 public:
  /** Default constructor.
      @param serial The serial instance this instance will use to
      communicate with the game. Usually "Serial".
      @param numRequests The number of request channels we'll allocate.
  */
  ObjectsInSpace(Stream &serial, int numRequests);

  /* Handshaking */
  /** Initialise the connection, and perform handshaking with the game.
      Note that this will set the speed and set up the serial connection,
      no need to call `Serial.begin()`.
  */
  bool begin();

  /* Syncing */
  /** Register a client command. See the protocol documentation for a
      full list of possible commands.
      @param command The command constant to register.
  */
  int registerCommand(OISCommand command);
  /** Register a request for boolean data. See the protocol documentation
      for a full list of possible data.
      @param command The data command to request.
      @param callback A callback function to handle the received data.
  */
  int registerBool(OISCommand command, boolCallback callback);
  /** Register a request for numeric data. See the protocol documentation
      for a full list of possible data.
      @param command The data command to request.
      @param callback A callback function to handle the received data.
  */
  int registerInt(OISCommand command, intCallback callback);
  /** Register a request for floating point data. See the protocol
      documentation for a full list of possible data.
      @param command The data command to request.
      @param callback A callback function to handle the received data.
  */
  int registerFloat(OISCommand command, floatCallback callback);
  /** Register a request for boolean data. This method passes data
      from the game directly to the callback, without further
      processing.
      @param command The data to request.
      @param callback A callback function to handle the received data.
  */
  int registerBoolRaw(OISCommand command, rawCallback callback);
  /** Register a request for numeric data. This method passes data
      from the game directly to the callback, without further
      processing.
      @param command The data to request.
      @param callback A callback function to handle the received data.
  */
  int registerIntRaw(OISCommand command, rawCallback callback);
  /** Register a request for floating point data. This method passes
      data from the game directly to the callback, without further
      processing.
      @param command The data to request.
      @param callback A callback function to handle the received data.
  */
  int registerFloatRaw(OISCommand command, rawCallback callback);
  /** Start active phase. This function tells the game that we have
      finished setting up commands and channels, and are ready to start
      sending and receiving game data.
  */
  void activate();

  /* Active phase */
  /** Send a command to the game. See the protocol documentation for a
      full list of possible commands.
      @param command The command constant to send.
  */
  void sendCommand(OISCommand command);
  /** Send a log message to the game. The game will treat this as a debug
      log, and add it to the game log file.
      @param message A string to be logged.
  */
  void sendDebug(char* message);
  /** Helper function to convert data received from the game to
      a floating point number.
  */
  float toFloat(int data);
  /** The main ObjectsInSpace loop. Call this regularly in your loop()
      function.
  */
  void update();
  #ifdef OIS_DEBUG
  /** An optional callback to receive error messages. To use this, add
      `#define OIS_DEBUG` to the top of your sketch, and create an
      OISErrorCallback function that will receive and process errors.
  */
  void errorHandler(OISErrorCallback callback);
  #endif
 private:
  // maximum number of request channels
  int _numRequests;
  // Our serial device
  Stream *_serial;

  OISState _streamState;
  // This will hold an array of callback functions
  AISRequest** _commandCallbacks;
  // Tracks _commandCallbacks index while filling the array in Syncing phase.
  int _callbackidx;

  char sendBuffer[bufSize];
  // This is arguably overkill for the receive buffer.
  char recvBuffer[bufSize];
  short recvidx = 0;

  void _readSerial();
  void _parseSerial();

  bool _validateCommand(OISCommand command);
  void _sendRequest(OISRequest request, OISCommand command);

  #ifdef OIS_DEBUG
  OISErrorCallback _errorCallback;
  #endif
};

#endif
